/***************************************************
 Zegar na bazie Arduino i płytki AVT-1616 (PCF8583,
 DS18B20, fotorezystor, 4xLED 7-segmentowy).
 _Przykład 1_:
   Proste wskazanie godziny i sekund, nieużywany
   kalendarz, fotorezystor i termometr.
****************************************************/
//biblioteka RTC
#include <PCF8583.h>
//biblioteka i2c
#include <Wire.h>
//adres PCF8583
PCF8583 p (0xA0);
//tymczasowe zmienne do nastawy czasu
byte tmpminuty, tmpgodziny, tmppozycja = 0;

//definicje wyprowadzeń portu PORT B
#define pb0  8
#define pb1  9
#define pb2  10
#define pb3  11
#define pb4  12
#define pb5  13
//definicje wyprowadzeń portu PORT D
#define pd0  0
#define pd1  1
#define pd2  2
#define pd3  3
#define pd4  4
#define pd5  5
#define pd6  6
#define pd7  7

//stan 0 - odczyt przycisków, 1=odczyt przycisków, 2...5=wyświetlanie liczby 1...4
byte numer_stanu = 0;
//bufor wyświetlacza LCD
byte bufor_led[4] = {0, 1, 2, 3};
//tryb pracy wyświetlacza
byte tryb_wyswietlania = 0;
//definicje segmentów dla instrukcji "AND"
#define segA  0xFE
#define segB  0xFD
#define segC  0xFB
#define segD  0xF7
#define segE  0xEF
#define segF  0xDF
#define segG  0xBF
#define segDP 0x7F
//tablica wzorców znaków do konwersji
const byte wzorce_znakow[11] = {
segA & segB & segC & segD & segE & segF,    //0
segB & segC,                                //1
segA & segB & segD & segE & segG,           //2
segA & segB & segC & segD & segG,           //3
segB & segC & segF & segG,                  //4
segA & segC & segD & segG & segF,           //5
segA & segC & segD & segE & segF & segG,    //6
segA & segB & segC,                         //7
segA & segB & segC & segD & segE & segF & segG,    //8
segA & segB & segC & segD & segF & segG,    //9
0xFF
};
//wyprowadzania załączające poszczególne cyfry (port PB)
const byte anody_cyfr[4] = {0xFE, 0xFD, 0xFB, 0xF7};
//aliasy wyprowadzeń portów
const int Buzzer =  13;
const int DP =  12; 
const int SW1 =  A1; 
const int SW2 =  A2;
const int sens_sw =  A0; 

/**************************************************
 inicjalizacja mikrokontrolera
 **************************************************/
void setup()
{
//wyjścia sterujace segmentami wyświetlacza
  pinMode(pd0, OUTPUT);
  pinMode(pd1, OUTPUT);
  pinMode(pd2, OUTPUT);
  pinMode(pd3, OUTPUT);
  pinMode(pd4, OUTPUT);
  pinMode(pd5, OUTPUT);
  pinMode(pd6, OUTPUT);
  pinMode(pd7, OUTPUT);
//wyłączenie wszystkich segmentów przez podanie "1"
  PORTD = 0xFF;
//wejście napięcia z fotorezystora
  pinMode(A0, INPUT);
//linia 1-wire
  pinMode(A3, INPUT);
//interfejs i2c
  pinMode(A4, INPUT);
  pinMode(A5, OUTPUT);
  digitalWrite(A5, HIGH);
//wyjśćie sterujące załączaniem cyfr
  pinMode(pb0, OUTPUT);
  pinMode(pb1, OUTPUT);
  pinMode(pb2, OUTPUT);
  pinMode(pb3, OUTPUT);
//wyprowadzenie sterujące dwukropkiem
  pinMode(pb4, OUTPUT);
//wyjście sterujace brzęczykiem
  pinMode(pb5, OUTPUT);
//ustawienie stanów początkowych wyprowadzeń
  PORTB = PORTB | 0x2F;
//konfigurowanie linii przycisków
  pinMode(SW1, INPUT);
  pinMode(SW2, INPUT);
//konfigurowanie przetwornika A/C; nieużywany w tym przykładzie
  analogReference(DEFAULT);
}

//konwersja i wyświetlenie liczby na LED, liczby są w zmiennej globalnej
//bufor_led; argumentem funkcji jest numer wyświetlanej pozycji
void wyswietl_LED(byte numer_pozycji)
{
//wyłączenie wszystkich cyfr
  PORTB = PORTB | 0x0F;
//konwersja liczby na znak do wyświetlenia, wyświetlenie znaku
//argument numer pozycji jest o 1 większy od indeksu tablicy, więc pomniejszamy go
//bufor_led zawiera liczby do wyświetlania, tablica wzorce_znaków pod numerem indeksu
//zawiera odpowiednie rozmieszczenie segmentów do wyświetlenia
  PORTD = wzorce_znakow[bufor_led[numer_pozycji-1]];
  PORTB = PORTB & anody_cyfr[numer_pozycji-1];
  delay(5);
}

//główna pętla programu
void loop()
{
/*program ma 6 stanów, pętla jest wykonywana nieskończenie, więc można użyć instrukcji
  "for", która będzie wykonywana w nieskończonej pętli poprawnie numerując stany
  - stan 0: obsługa klawiszy i nastaw,
  - stan 1: odczyt zegara RTC,
  - stan 2...5: wyświetlanie poszczególnych cyfr.
  Numery stanów zawiera zmianna "numer_stanu", są one rozpatrywane w klauzuli switch-case.
*/
  for (numer_stanu = 0; numer_stanu < 6; numer_stanu++)
  {
    switch (numer_stanu)
    {
//obsługa klawiszy
      case 0:
  //przełączanie trybów wyświetlania za pomocą przycisku SW1, który działa
  //inaczej w trybach wyświetlania 0 (wyswietlanie czasu) i 1 (wyświetlanie sekund),
  //a inaczej w trybie 3 (nastaw)
        if (tryb_wyswietlania != 3)
        {
  //jeśli tryb wyświetlania jest różny od 3, to zegar jest w trybie wskazań czasu
  //jeśli jest równy 3, to w trybie nastaw
          if (digitalRead(SW1) == 0)
          {
            tryb_wyswietlania++;
            while (digitalRead(SW1) == 0);
  //poza nastawami zmianna tryb_wyświetlania nie może być większa od 1
            if (tryb_wyswietlania > 1) tryb_wyswietlania = 0;
          }
  //jeśli w trybie nastaw, to klawisze SW1 i SW2 mają inne funkcje
  //SW1 powoduje przejście od nastawy do nastawy, SW2 powoduje zmianę nastawy
          if (digitalRead(SW2) == 0 && tmppozycja == 0)
          {
  //przepisanie godziny i minut z RTC do zmiennych tymczasowych
            tmpminuty = p.minute;
            tmpgodziny = p.hour;
            tryb_wyswietlania = 3;
  //załączenie na stałe dwukropka
            digitalWrite(pb4, LOW);
  //wybranie minut do edycji (minuty tmppozycja=1, godziny tmppozycja=2,
  //start zegara tmppozycja>= 3)
            tmppozycja = 1;
  //oczekiwanie na zwolnienie SW2
            while (digitalRead(SW2) == 0);
          }
        } else
        {
  //sw2 & poz=1 - zmiana nastawy minut
          if (digitalRead(SW2) == 0)
          {
            if (tmppozycja == 1)
            {
  //zmiana "tymczasowych" minut
              tmpminuty++;
              if (tmpminuty > 59) tmpminuty = 0;
  //reakcja na pojedyncze przyciśnięcie, więc oczekiwanie na zwolnienie SW2
              while (digitalRead(SW2) == 0);
            }
  //sw2 & poz=2 - zmiana nastawy godzin
            if (tmppozycja == 2)
            {
  //zmiana "tymczasowych" godzin
              tmpgodziny++;
              if (tmpgodziny > 23) tmpgodziny = 0;
  //reakcja na pojedyncze przyciśnięcie, więc oczekiwanie na zwolnienie SW2
              while (digitalRead(SW2) == 0);
            }
  //sw2 & poz=3 - zmiana nastawy RTC
            if (tmppozycja >= 3)
            {
  //sygnalizacja dźwiękowa przy ustawieniu godziny (PB5=poziom niski)
              digitalWrite(pb5, LOW);
  //zapisanie nastaw do struktury czasu
              p.hour = tmpgodziny;
              p.minute = tmpminuty;
              p.second = 0;
  //ustawienie czasu
              p.set_time();
  //załączenie normalnego trybu pracy zegara
              tmppozycja = tryb_wyswietlania = 0;
  //oczekiwanie na zwolnienie SW2
              while (digitalRead(SW2) == 0);
  //wyłączenie sygnalizacji dźwiękowej
              digitalWrite(pb5, HIGH);
            }
          }
  //sw1 - następna pozycja nastaw
          if (digitalRead(SW1) == 0)
          {
            tmppozycja++;
            while (digitalRead(SW1) == 0);
          }
        }
        break;
//odczyt zegara
      case 1:
        p.get_time();
//wyświetlenie czasu, tryb 0
        if (tryb_wyswietlania == 0)
        {
          bufor_led[3]=  p.minute % 10;
          bufor_led[2] = p.minute / 10;
          bufor_led[1]=  p.hour % 10;
          bufor_led[0] = p.hour / 10;
          if (bufor_led[0] == 0) bufor_led[0] = 10;  //wyłączanie zera nieznaczącego
          if (p.second%2 == 0) digitalWrite(pb4, HIGH);  //sterowanie dwukropkiem (PB4)
            else digitalWrite(pb4, LOW);
        } else
//wyświetlenie sekund, tryb 1
          if (tryb_wyswietlania == 1)
          {
            bufor_led[3] = p.second % 10;
            bufor_led[2] = p.second / 10;
            bufor_led[1] = bufor_led[0] = 10;
            digitalWrite(pb4, LOW);
          } else
//wyświetlanie w czasie nastaw, tryb 3
            if (tryb_wyswietlania == 3)
            {
              bufor_led[3] = tmpminuty % 10;
              bufor_led[2] = tmpminuty / 10;
              bufor_led[1] = tmpgodziny % 10;
              bufor_led[0] = tmpgodziny / 10;
            }
        break;
//wyświetlenie cyfry 1 (pierwsza od prawej)
      case 2:
        wyswietl_LED(1);
        break;
//wyświetlenie cyfry 2
      case 3:
        wyswietl_LED(2);
        break;
//wyświetlenie cyfry 3
      case 4:
        wyswietl_LED(3);
        break;
//wyświetlenie cyfry 4
      case 5:
        wyswietl_LED(4);
        break;
//wyświetlenie cyfry 4 (pierwsza od lewej)
      default:
        break;
    }
  }
}

